


//                     123456789A123456789B  // a screen shown after startup
#define SPLASHSCREEN0 "** Modbus LCD **"
#define SPLASHSCREEN1 "     V1.0.0"
#define SPLASHSCREEN2 ""
#define SPLASHSCREEN3 ""

constexpr byte splashscreenTimeout = 10;     // in seconds. 0 = no splash screen

/*
   show some static information on LCD
*/
void showSplashscreen()
{
  lcd.clear();
  lcd.print(F(SPLASHSCREEN0));
  lcd.setCursor(0, 1);
  lcd.print(F(SPLASHSCREEN1));
  if (rows > 2)
  {
    lcd.setCursor(0, 2);
    lcd.print(F(SPLASHSCREEN2));
    lcd.setCursor(0, 3);
    lcd.print(F(SPLASHSCREEN3));
  }
}


/*
   show some static information on LCD
*/
void showConfig()
{
  lcd.clear();
  lcd.print(F("Addr:")); lcd.print(modbus_server_id);

  lcd.setCursor(0, 1);
  lcd.print(F("Baud:")); lcd.print(modbus_baud);
  lcd.write(' ');
  switch (modbus_format)                 // from HardwareSerial.h - but only the most relevant
  {
    case 0x04 : lcd.print(F("7N1")); break;
    case 0x06 : lcd.print(F("8N1")); break;
    case 0x0C : lcd.print(F("7N2")); break;
    case 0x0E : lcd.print(F("8N2")); break;
    case 0x24 : lcd.print(F("7E1")); break;
    case 0x26 : lcd.print(F("8E1")); break;
    case 0x2C : lcd.print(F("7E2")); break;
    case 0x2E : lcd.print(F("8E2")); break;
    case 0x34 : lcd.print(F("7O1")); break;
    case 0x36 : lcd.print(F("8O1")); break;
    case 0x3C : lcd.print(F("7O2")); break;
    case 0x3E : lcd.print(F("8O2")); break; // ok now I understand the logik ;-)
    default:
      lcd.print(modbus_format);
  }
}

class Indicator {                                // all indicators react very similar - therefore a class
  private:
    uint32_t previousMillis;                     // last time shown
    bool active = false;                         // indicator is shown
    const byte character;                        // the character to be displayed
    const byte col;                              // on which column
    const byte row;                              // on which row
    const uint16_t interval;                     // how long visible (after that time, the indicator will disapear)
  public:
    Indicator(byte character, byte col, byte row, uint16_t interval = 512) :
      character(character), col(col), row(row), interval(interval) {}
    void onIfNotActive() {       
      if (active == false)
      {
        lcd.setCursor(col, row);
        lcd.write(character);
        previousMillis = millis();
      }
      active = true;
    }
    void on() {
      active = true;
      lcd.setCursor(col, row);
      lcd.write(character);
      previousMillis = millis();
    }
    void off() {
      if (active == true)
      {
        lcd.setCursor(col, row);
        lcd.write(' ');
      }
      active = false;
    }
    void display() {
      lcd.setCursor(col, row);
      active ?  lcd.write(' ') : lcd.write(character);
    }
    void update() {                              // to be called in loop()
      if (active == true && interval > 0)
      {
        if (millis() - previousMillis > interval)
        {
          off();
        }
      }
    }
};

Indicator indicatorMasterTimeout('x', 15, 1, 0);    // character, col, row, no timeout

/*
   time management for the LCD.
   currently it only deletes the splash screen after startup
*/
void lcdTimer()
{
  if (mslrLcdWasUpdated == false && millis() > splashscreenTimeout * 1000UL) // delete splash screen
  {
    lcd.clear();
    mslrLcdWasUpdated = true;
    indicatorMasterTimeout.display();          // assumption, nothing received yet
  }
}
