/*******************************************************
  LCD Modbus Slave (Receiver) - Hello World

  description
  - Simple sketch how to use receive data on a Modbus Slave and display data on LCD

  structure of this sketch: 
  - hardware definitions for the LCD
  - definitions for the Serial interface - either Software or Hardware and if necessary the TX enable pin
  - definition of the Modbus library

  hardware
  - a LCD connected to the Arduino
  - a MAX485-TTL adapter 

  by noiasca
  2021-10-29 OK

 *******************************************************/

/* ******************************************************            
 * LCD 
 * *************************************************** */            

#include <NoiascaLiquidCrystal.h>      // download library from https://werner.rothschopf.net/202009_arduino_liquid_crystal_intro.htm
#include <NoiascaHW/lcd_4bit.h>        // parallel interface, 4bit - but you can use any other interface from that library also.
constexpr byte cols = 16;                  // columns/characters per row
constexpr byte rows = 2;                   // how many rows
constexpr byte rs = 8;                     // RS pin
constexpr byte en = 9;                     // EN pin
constexpr byte d4 = 4;
constexpr byte d5 = 5;
constexpr byte d6 = 6;
constexpr byte d7 = 7;
constexpr byte bl = 10;                    // backlight pin
constexpr t_backlightPol blType = FAULTY;  // set to either POSITIVE, NEGATIVE, FAULTY (for some keypad shields)
LiquidCrystal_4bit lcd(rs, en, d4, d5, d6, d7, bl, blType, cols, rows); // Standard UTF-8 converter


/* *******************************************************            
 * Serial Interface
 * **************************************************** */    
// if you don't have enough HW Serial (i.e. on an UNO)
// you are forced to use SoftwareSerial or AltSoftSerial
#include <SoftwareSerial.h>
SoftwareSerial modbusSerial(2, 3);               // RX, TX

// On a Mega you can simply use 
// a Reference to an existing HW Serial:
// HardwareSerial &modbusSerial = Serial3;

// If you don't need debugging output, you can use
// a reference to Serial
// HardwareSerial &modbusSerial = Serial;

constexpr byte modbus_format = SERIAL_8N1;   // leave it on SERIAL_8N1 for SoftSerial
constexpr uint32_t modbus_baud = 19200;      // use only slow speeds with SoftSerial
constexpr byte modbus_enable_pin = 11;       // The GPIO used to control the MAX485 TX pin. Set to 255 if not needed (i.e. a selfsensing module or RS232).

/* **************************************************** *            
 * Modbus 
 * **************************************************** */    

#include <SimpleModbusSlave.h>               // Modbus Slave library (low level RS484 communication)
#include <modbus_common.h>                   // predefined Modbus LCD registers 
#include <modbus_slave_lcd_procedural.h>                // Modbus Slave handling of incomming LCD commands
constexpr byte modbus_slave_id = 2;          // Modbus address of this slave

// FC3/FC16 holding registers - not used in this example
enum
{
  HOLDING_REGS_SIZE  // leave this one
};
unsigned int holdingRegs[HOLDING_REGS_SIZE]; // FC3/FC16 register array

/* **************************************************** *            
 * Arduino default setup and loop
 * **************************************************** */   
void setup()
{
  Serial.begin(115200);                     // only used for debugging
  Serial.println(F("Modbus LCD Slave 0202"));
  
  modbusSerial.begin(modbus_baud);          // start the Serial for Modbus
  modbus_configure(&modbusSerial, modbus_baud, modbus_format, modbus_slave_id, modbus_enable_pin, HOLDING_REGS_SIZE, holdingRegs);

  lcd.begin();                              // initialize the display
  lcd.backlight();
  lcd.setCursor(0, 0);
  lcd.print(F("Modbus LCD Slave"));         // print a simple message on LCD
}

void loop()
{
  modbus_update();   // call modbus in loop
  // do other things non blocking in loop
}
