/*
   Verschiedene Ansätze
   2021-10-28 funktioniert
*/

#include <ModbusMaster.h>
#include <modbus_common.h>

constexpr byte wait = 2;  // use 5 for 9600 Baud - set some delay time

/*!
  We're using a MAX485-compatible RS485 Transceiver.
  Rx/Tx is hooked up to the hardware serial port at 'Serial'.
  The Data Enable and Receiver Enable pins are hooked up as follows:
*/
#define MAX485_DE      5

// instantiate ModbusMaster object
ModbusMaster node;

void preTransmission()
{
  digitalWrite(MAX485_DE, HIGH);
}

void postTransmission()
{
  digitalWrite(MAX485_DE, LOW);
}


size_t actBuffer = 0;

size_t write(char c)   // add to Buffer
{
  node.setTransmitBuffer(actBuffer, c);
  Serial.println(actBuffer);
  actBuffer++;
  return 1;
}

size_t printNumber(unsigned long n, uint8_t base)
{
  char buf[8 * sizeof(long) + 1]; // Assumes 8-bit chars plus zero byte.
  char *str = &buf[sizeof(buf) - 1];

  *str = '\0';

  // prevent crash if called with base == 1
  if (base < 2) base = 10;

  do {
    char c = n % base;
    n /= base;

    *--str = c < 10 ? c + '0' : c + 'A' - 10;
  } while (n);

  return write(str);
}

size_t print(char c)
{
  return write(c);
}

size_t print(long n, int base = 10)
{
  if (base == 0) {
    return write(n);
  } else if (base == 10) {
    if (n < 0) {
      int t = print('-');
      n = -n;
      return printNumber(n, 10) + t;
    }
    return printNumber(n, 10);
  } else {
    return printNumber(n, base);
  }
}

int begin(byte a = 0, byte b = 0, byte c = 0)
{
  (void)a;  //ignore any parameter
  (void)b;
  (void)c;
  byte result = node.writeSingleRegister(addrBegin, 0);
  delay(wait);
  return result;
}

int clear()
{
  byte result = node.writeSingleRegister(addrClear, 0);
  delay(wait);
  return result;
}

int home()
{
  byte result = node.writeSingleRegister(addrHome, 0);
  delay(wait);
  return result;
}

int blink()
{
  byte result = node.writeSingleRegister(addrBlink, 255);
  delay(wait);
  return result;
}

int noBlink()
{
  byte result = node.writeSingleRegister(addrBlink, 0);
  delay(wait);
  return result;
}

int cursor()
{
  byte result = node.writeSingleRegister(addrCursor, 255);
  delay(wait);
  return result;
}

int noCursor()
{
  byte result = node.writeSingleRegister(addrCursor, 0);
  delay(wait);
  return result;
}

int setCursor(byte col, byte row)
{
  uint16_t out = ((uint16_t)col) << 8 | row;
  byte result = node.writeSingleRegister(addrSetCursor, out);
  delay(wait);
  return result;
}

int backlight(void) {
  return setBacklight(255);
}

int noBacklight(void) {
  return setBacklight(0);
}

int setBacklight(uint8_t newVal) {
  byte result = node.writeSingleRegister(addrSetBacklight, newVal);
  delay(wait);
  return result;
}


void splash()
{
  node.setTransmitBuffer(0, (uint16_t)'M' << 8 | 'a');
  node.setTransmitBuffer(1, (uint16_t)'s' << 8 | 't');
  node.setTransmitBuffer(2, (uint16_t)'e' << 8 | 'r');
  node.setTransmitBuffer(3, (uint16_t)' ' << 8 | 'o');
  node.setTransmitBuffer(4, (uint16_t)'n' << 8 | 'l');
  node.setTransmitBuffer(5, (uint16_t)'i' << 8 | 'n');
  node.setTransmitBuffer(6, (uint16_t)'e' << 8 | '-');
  node.writeMultipleRegisters(1000, 7);
  delay(wait);
}


void splash2()
{
  node.setTransmitBuffer(0, 'M');
  node.setTransmitBuffer(1, 'a');
  node.setTransmitBuffer(2, 's');
  node.setTransmitBuffer(3, 't');
  node.setTransmitBuffer(4, 'e');
  node.setTransmitBuffer(5, 'r');
  node.setTransmitBuffer(6, ' ');
  node.setTransmitBuffer(7, 'o');
  node.setTransmitBuffer(8, 'n');
  node.setTransmitBuffer(9, 'l');
  node.setTransmitBuffer(10, 'i');
  node.setTransmitBuffer(11, 'n');
  node.setTransmitBuffer(12, 'e');
  node.writeMultipleRegisters(1000, 13);
  delay(wait);
}


#include <NoiascaLiquidCrystal.h>
//#include <lcd_modbus_print.h>

class MyLcd : public LiquidCrystal_dummy
{
  public :
    MyLcd() : LiquidCrystal_dummy(2, 16) {}

    size_t write(uint8_t value) {      // decided against inline
      DEBUG_PRINTLN(F("Modbus write"));
      byte result = node.writeSingleRegister(addrWrite, value);
      Serial.print(F("Result ")); Serial.println(result, HEX);
      delay(wait);
      return 1;
    }

};

MyLcd lcd;

void setup()
{
  Serial.begin(115200);
  Serial.println(F("MEGA Half Duplex"));

  pinMode(MAX485_DE, OUTPUT);
  // Init in receive mode
  digitalWrite(MAX485_DE, LOW);

  // Modbus communication baud
  Serial3.begin(19200);

  // Modbus slave ID 1
  node.begin(2, Serial3);
  // Callbacks allow us to configure the RS485 transceiver correctly
  node.preTransmission(preTransmission);
  node.postTransmission(postTransmission);

  //clear();
  begin();
  backlight();

  splash();
  
}

bool state = true;



void loop()
{
  uint8_t result;
  uint16_t data[6];

  state = !state;

  setCursor(0,0);
  lcd.write('X');
  lcd.print("abcd");



  
  //setBacklight(state);

  // Toggle the coil at address 0x0002 (Manual Load Control)
  //result = node.writeSingleCoil(0x0002, state);

  //result = node.writeSingleRegister(10, state);
  //Serial.print(F("Result ")); Serial.println(result, HEX);
  //delay(wait);

  //clear();

  setCursor(10, 1);

  result = node.writeSingleRegister(addrWrite, state + '0');
  Serial.print(F("Result ")); Serial.println(result, HEX);
  delay(wait);

  //result = node.writeSingleRegister(addrWriteOld, state);
  //Serial.print(F("Result ")); Serial.println(result, HEX);
  //delay(wait);

  setCursor(3, 1);

  byte now = (byte)(millis() / 1000);
  node.setTransmitBuffer(0, now / 10000 % 10000 + '0');
  node.setTransmitBuffer(1, now / 1000 % 1000 + '0');
  node.setTransmitBuffer(2, now / 100 % 100 + '0');
  node.setTransmitBuffer(3, now / 10 % 10 + '0');
  node.setTransmitBuffer(4, now % 10 + '0');
  result = node.writeMultipleRegisters(addrStreamExpand, 5);
  Serial.print(F("Result ")); Serial.println(result, HEX);
  delay(wait);

  //NOK
  //print(12345L);
  //finalize:
  //Serial.println(actBuffer);
  //result = node.writeMultipleRegisters(1000, actBuffer - 1);
  //actBuffer = 0;
  //Serial.print(F("Result ")); Serial.println(result);
  //delay(wait);

  /*
    // Read n registers starting at m)
    byte count = 6;
    result = node.readHoldingRegisters(0, count);
    if (result == node.ku8MBSuccess)
    {
    for (int i = 0; i < count; i++)
    {
      Serial.print(node.getResponseBuffer(i), HEX);
      Serial.print(" ");
    }
    Serial.println();
    }
    else
    {
    Serial.print(F("Error ")); Serial.println(result, HEX);
    }
  */

  delay(3000);
}
