/*
   Modbus LCD Master

   using of Modbus LCD based on Print.h
   2021-10-28 OK
*/

#include <ModbusMaster.h>
#include <modbus_common.h>

/*!
  We're using a MAX485-compatible RS485 Transceiver.
  Rx/Tx is hooked up to the hardware serial port at 'Serial'.
  The Data Enable and Receiver Enable pins are hooked up as follows:
*/

constexpr byte modbus_format = SERIAL_8N1;   // if you can use Hardware Serial, prefered value is SERIAL_8N2
constexpr uint32_t modbus_baud = 19200;      // use slow speeds with SoftSerial
constexpr byte modbus_enable_pin = 5;        // The GPIO used to control the MAX485 TX pin. Set to 255 if you are not using RS485 or a selfsensing adapter

// instantiate ModbusMaster object
ModbusMaster node;
constexpr byte modbus_slave_address = 2;     // Address of the slave with LCD
constexpr byte wait = 2;

/*
    Modbus Master callbacks
*/
void preTransmission()
{
  digitalWrite(modbus_enable_pin, HIGH);
}

void postTransmission()
{
  digitalWrite(modbus_enable_pin, LOW);
}

// replace LCD driver.
// This LCD interface connects to Modbus via Serial
#include <lcd_modbus_legacy.h>      // this lcd class inherits from default print 
LiquidCrystal_modbus lcd(node);    // would be nice if we could handove the slave address


void splash()
{
  node.setTransmitBuffer(0, (uint16_t)'M' << 8 | 'a');
  node.setTransmitBuffer(1, (uint16_t)'s' << 8 | 't');
  node.setTransmitBuffer(2, (uint16_t)'e' << 8 | 'r');
  node.setTransmitBuffer(3, (uint16_t)' ' << 8 | 'o');
  node.setTransmitBuffer(4, (uint16_t)'n' << 8 | 'l');
  node.setTransmitBuffer(5, (uint16_t)'i' << 8 | 'n');
  node.setTransmitBuffer(6, (uint16_t)'e' << 8 | '-');
  node.setTransmitBuffer(7, (uint16_t)'0' << 8 | '\0');
  node.writeMultipleRegisters(1000, 8);
  delay(wait);
}

void splash2()
{
  node.setTransmitBuffer(0, 'M');
  node.setTransmitBuffer(1, 'a');
  node.setTransmitBuffer(2, 's');
  node.setTransmitBuffer(3, 't');
  node.setTransmitBuffer(4, 'e');
  node.setTransmitBuffer(5, 'r');
  node.setTransmitBuffer(6, ' ');
  node.setTransmitBuffer(7, 'o');
  node.setTransmitBuffer(8, 'n');
  node.setTransmitBuffer(9, 'l');
  node.setTransmitBuffer(10, 'i');
  node.setTransmitBuffer(11, 'n');
  node.setTransmitBuffer(12, 'e');
  node.setTransmitBuffer(13, '-');
  node.setTransmitBuffer(14, '1');
  node.setTransmitBuffer(15, '6');
  node.writeMultipleRegisters(addrStreamExpand, 16);
  delay(wait);
}

/* **************************************************** *
   Arduino default setup and loop
 * **************************************************** */
void setup()
{
  Serial.begin(115200);
  Serial.println(F("MEGA Half Duplex"));

  // Modbus communication baud
  Serial3.begin(modbus_baud);

  // Init enable pins for modbus master library
  pinMode(modbus_enable_pin, OUTPUT);
  digitalWrite(modbus_enable_pin, LOW);

  // Modbus slave and Serial modification
  node.begin(modbus_slave_address, Serial3);

  // Callbacks allow us to configure the RS485 transceiver correctly
  node.preTransmission(preTransmission);
  node.postTransmission(postTransmission);

  lcd.begin();
  lcd.setDelayPostWrite(wait); // default 5 for 9600 Baud - set some delay time
  //lcd.clear();
  lcd.backlight();

  lcd.print(F("Master online"));
}

void loop()
{
  //uint16_t data[6];
  static bool state = true;
  state = !state;

  uint32_t startMicros, endMicros;

  lcd.setCursor(0, 0);
  startMicros = micros();
  lcd.print("123456789a123456"); // 270680 µsec
  endMicros = micros();
  Serial.print("time: "); Serial.println(endMicros - startMicros);
  delay(500); // only to make the text readable 

  lcd.setCursor(0, 0);
  startMicros = micros();
  splash(); // 40724 µsec
  endMicros = micros();
  Serial.print("time: "); Serial.println(endMicros - startMicros);
  delay(500); // only to make the text readable 

  lcd.setCursor(0, 0);
  startMicros = micros();
  splash2(); // 61584 µsec
  endMicros = micros();
  Serial.print("time: "); Serial.println(endMicros - startMicros);

  lcd.setCursor(0, 1);
  lcd.write('X' + state);
  lcd.print("abc");
  lcd.setCursor(11, 1);

  //lcd.setBacklight(state);
  lcd.setCursor(5, 1);
  lcd.print(millis() / 1000);

  // The example slave offers: 3 Registers: ADC0, DI12, DO13

  // write to slave
  node.writeSingleRegister(2, state);
  delay(5); // give the slave time to settle
  // read from slave registers
  uint8_t result;
  result = node.readHoldingRegisters(0, 2);
  if (result == node.ku8MBSuccess)
  {
    Serial.print(node.getResponseBuffer(0));  // ADC0
    Serial.print("\t");
    Serial.print(node.getResponseBuffer(1));  // DI12
    Serial.println();
  }
  else
  {
    Serial.print("result=");  Serial.println(result, HEX);
  }


  // retrive a value from the slave:

  delay(3000);
}
